import { ifElse, is, equals, length, not, flatten, reduce, identity, isArrayLike } from 'ramda'
import { dynamicNode } from 'baobab'
import listPath from './listPath'
import isPath from './isPath'

const makePath = ifElse(is(String), listPath, identity)

const prepareArr = (acc, x) => {

  if (is(String, x) && true !== isPath(x)) {
    throw `dynamicNode needs json paths "${x}"`
  }

  acc.push(makePath(x))
  return acc
}

const dynamic = function () {
  let args = Array.prototype.slice.call(arguments)

  if (!equals(2, length(args))) {
    throw new Error(`dynamicNode needs 2 params but "${length(args)}" were specified`)
  }

  if (not(isArrayLike(args[0]))) {
    throw new Error(`dynamicNode parameters are invalid. First param needs to be an array. "${args}"`)
  }

  if (!is(Function, args[1])) {
    throw new Error(`dynamicNode paramters are invalid. Second param needs to be a function.  "${args}"`)
  }

  args = flatten(args)

  args.forEach(x => {
    if (is(String, x) && !isPath(x)) {
      throw `dynamicNode parameters are invalid. Param is not a valid path. "${args}"`
    }
  })

  args = reduce(prepareArr, [], args)

  let node = dynamicNode.apply(null, args)

  return node
}

export default dynamic
