import { cond, contains, pipe, equals, is, nthArg, always, T } from 'ramda'
import isPath from './isPath'

/**
 * Tests if a given value corresponds to the given operation
 *
 * e.g. ADD should not have an undefined value
 *
 * e.g. MOVE should have a path value
 *
 * @function
 * @param {string} operation - The operation that will decide the type of value
 * @param {string} value - The value that will be tested according to specifications
 * @returns {true|string} result - ** true ** if the value is correct or a ** string ** with the error
 * @example
 *
 * isValueForOperation('add', 'foo') // true
 * isValueForOperation('remove', null) // true - on remove the value is simply ignored
 * isValueForOperation('move', 'foo') // The path "foo!" did not match the encoded path regexp [regular expression]
 */
const isValueForOperation = cond([
    [
      (op, value) => contains(op, ['add', 'replace', 'test']) && equals(undefined, value),
      always('The value or from path provided must be different than undefined')
    ],
    [ pipe(nthArg(0), equals('remove')), T ],
    [
      (op, value) => contains(op, ['move', 'copy']) && pipe(isPath, is(String))(value),
      pipe(nthArg(1), isPath)
    ],
    [T, T]
])

export default isValueForOperation
